class DefaultHandler:

    @staticmethod
    def origin2flow(request_data):
        return request_data

    @staticmethod
    def flow2origin(flow_data):
        return flow_data

    @staticmethod
    def origin2string(request_data):
        return request_data
