STATUS_INFINITE_PROXY_LOOP = '901 INFINITE PROXY LOOP'
STATUS_CODE_INFINITE_PROXY_LOOP = 901

STATUS_CAN_NOT_HANDLE_REQUEST = '904 CAN NOT HANDLE REQUEST'
STATUS_CODE_CAN_NOT_HANDLE_REQUEST = 904

STATUS_PROXY_ERROR = '905 PROXY ERROR'
STATUS_CODE_PROXY_ERROR = 905
